/* **************************************************************************** 
  This file is part of the kde-game 'kjumpingCube'

  Copyright (C) 1998 by Matthias Kiefer <Matthias.Kiefer@stud.uni-karlsruhe.de>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  *******************************************************************************/  

#include <stdlib.h>
#include <math.h>
#include <qstring.h>
#include <kapp.h>
#include <time.h>

#include "engine.h"
#include "cubesim.h"


Engine::Engine(int value)
{
   setSkill(1);
   stopped=FALSE;
   active=FALSE;
   currentLevel=0;
   if(!value)      
      init(time(NULL));
}


void Engine::init(int value)
{
   srand48(value);
   
}

int Engine::setSkill(int newSkill)
{  
   int oldSkill=skill;
   skill=newSkill;
   maxLevel=newSkill; 
   
   return oldSkill; 
}

int Engine::getSkill()
{
   return skill;
}
		


bool Engine::getHint(int& row, int& column, int player ,CubeSimBox box)
{
   int i,j;
   // if more than one cube has the same rating these arrays are used to select
   // one
   QByteArray rows(box.size*box.size);
   QByteArray columns(box.size*box.size);
 
   active=TRUE;
   stopped=FALSE;
   currentPlayer=player;
        
   int **worth=new int*[box.size];
   for(i=0;i<box.size;i++)
      worth[i]=new int[box.size];

    // Zge bewerten
     
   for(i=0;i<box.size;i++)
      for(j=0;j<box.size;j++)
      {
	 // Thinking process stopped
	 if(stopped)
	 {
	    active=FALSE;
	    return FALSE;
	 }
	 
	 worth[i][j]=doMove(i,j,player,box);
      }
      
 
   // Maximum finden
   int max=-(int)pow(2,sizeof(int)*8-1);  // Maximum auf kleinst mglichen Wert setzen
  
   
   for(i=0;i<box.size;i++)
     for(j=0;j<box.size;j++)
     {
	if(worth[i][j]>max )
	   if(box.cube(i,j).getOwner()==player || box.cube(i,j).getOwner()==0)
	       max=worth[i][j];
     }
   
 	QString t;  
	t.sprintf("max: %d",max);
        debug(t);
	
     // Maximum mehrmals vorhanden?
     int counter=0;
     for(i=0;i<box.size;i++)
        for(j=0;j<box.size;j++)
        {
	   if(worth[i][j]==max)  
              if(box.cube(i,j).getOwner()==player || box.cube(i,j).getOwner()==0)
	      {
	         rows[counter]=i;
	         columns[counter]=j;
	         counter++;  
	      }
        }      

	t.sprintf("anzahl max: %d",counter);
	debug(t);
	
	// vorlufig
	if(counter==0)
	{
	   active=FALSE;
	   return FALSE;
	}
	
	if(counter>1)
	{
	   counter=1+int(drand48()*counter);
	   
	}
	   
	row=rows[counter-1];
	column=columns[counter-1];
	
	active=FALSE;
	return TRUE;
}


void Engine::stop()
{
   stopped=TRUE;
}


bool Engine::isActive()
{
   return active;
}


int Engine::doMove(int row, int column, int player , CubeSimBox box)
{
   int worth=0;  
   currentLevel++;
 
   if(currentLevel < maxLevel)
   { 
      
      if(!box.simulateMove(row,column,player))
      {
	 currentLevel--;
	 return 0;
      }
      
      if(box.playerWon(player))
      {
	 currentLevel--;
	
	 return (int)pow(box.size*box.size,(maxLevel-currentLevel-1))*box.assessField(currentPlayer);	
      }
            
      int i,j;
      
      // naechster Zug anderer Spieler
      player=(player==1)? 2 : 1;
      for(i=0;i<box.size;i++)
      {
	 kapp->processEvents();
	 for(j=0;j<box.size;j++)
         {
	    // Thinking process stopped
	    if(stopped)
	    {
	       currentLevel--;
	       return 0;
	    }
	    
	    worth+=doMove(i,j,player,box);
	 }
      }
       
      currentLevel--;
      return worth;
   }
   else
   {
      currentLevel--;
      box.simulateMove(row,column,player);
      
      return box.assessField(currentPlayer);
   }
   
}


